# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.mail import send_mail
from django.core.mail import EmailMultiAlternatives
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404, redirect
from django.template.loader import render_to_string
from lib.access_auth import permission_needed
from lib.tools import erreur, confirmation
from logement.models import Logement, LogementPhoto, NB_AVANT_OBSOLETE
from logement.forms import (
    SearchForm,
    LogementForm,
    LogementPublicForm,
    LogementPhotoForm,
    LogementPhotoFormSet,
)
from portailuser.models import Ecole
from pages.models import PageWeb
from django.utils.safestring import mark_safe
from portail.settings import EMAIL_LOCA7
from lib.geo_code import BING_MAPS_API_KEY


def accueil(request):
    if request.ecole != Ecole.n7 or (request.user.is_authenticated and request.user.ecole != Ecole.n7):
        return erreur(
            request,
            "Désolé, le service Loca7 est réservé aux étudiants de l’ENSEEIHT.",
            "Accès refusé",
        )

    # Redirection vers loca7.fr
    return redirect("https://loca7.fr")

    if not request.user.is_authenticated and not request.session.get("logt_confirm", False):
        request.session["logt_verif_confirm"] = True
        return render(request, "logement/confirmation.html")

    if request.method == "POST":
        form = SearchForm(request.POST)

        if form.is_valid():
            min_surface = form.cleaned_data["min_surface"]
            max_surface = form.cleaned_data["max_surface"]
            max_loyer = form.cleaned_data["max_loyer"]
            meuble = form.cleaned_data["meuble"]

            meuble = True if meuble == "True" else False if meuble == "False" else None

            query_dict = {}
            aetefiltre = False
            if min_surface:
                query_dict["surface__gte"] = min_surface
                aetefiltre = True
            if max_surface:
                query_dict["surface__lte"] = max_surface
                aetefiltre = True
            if max_loyer:
                query_dict["loyer__lte"] = max_loyer
                aetefiltre = True
            if meuble is not None:
                query_dict["meuble"] = meuble
                aetefiltre = True

            query_dict["statut__exact"] = 0

            logements = Logement.objects.filter(**query_dict).order_by("pub_date")

            # Affichage user-friendly de la recherche
            if min_surface is not None and max_surface is not None:
                surface_text = f"entre {min_surface} et {max_surface} m²"
            elif min_surface is not None:
                surface_text = f"plus de {min_surface} m²"
            elif max_surface is not None:
                surface_text = f"moins de {max_surface} m²"
            else:
                surface_text = "surface quelconque"

            if meuble:
                meuble_text = ", meublé"
            elif meuble is False:
                meuble_text = ", non meublé"
            else:
                meuble_text = ""

            if max_loyer is None:
                loyer_text = ""
            else:
                loyer_text = f", moins de {max_loyer} €"

            if not aetefiltre:
                desc_affichage = "Tous les logements"
            else:
                desc_affichage = "Résultats pour la recherche : " + surface_text + meuble_text + loyer_text

            c = {
                "logements": logements,
                "form": form,
                "desc_affichage": desc_affichage,
                "api_key": BING_MAPS_API_KEY,
            }

            return render(request, "logement/recherche.html", c)
        else:
            c = {
                "form": form,
                "api_key": BING_MAPS_API_KEY,
                "invalid_form": True,
            }
            return render(request, "logement/recherche.html", c)

    else:
        # Texte d'accueil
        try:
            page = PageWeb.objects.get(ecole=request.ecole, adresse="logement/accueil")
            texte_accueil = page.contenu.as_html()
            lien_page = "/gestion/pages/" + str(page.id)
            message_accueil_visible = True
        except PageWeb.DoesNotExist:
            if request.user.is_authenticated and request.user.is_staff:
                texte_accueil = "Pas (encore) de texte ici..."
                lien_page = "/gestion/pages/nouvelle?adresse=/logement/accueil"
                message_accueil_visible = True

            else:
                texte_accueil = ""
                lien_page = ""
                message_accueil_visible = False

        gestion_logement = False
        gestion_page = False
        if not request.user.is_anonymous:
            gestion_logement = request.user.check_permission(request, "gerer_logements")
            gestion_page = request.user.check_permission(request, "gerer_pages")

        # Formulaire
        form = SearchForm()
        # 10 logements les plus récents
        logements = Logement.objects.filter(statut=0).order_by("-date_maj")[:10]

        c = {
            "logements": logements,
            "form": form,
            "texte_accueil": texte_accueil,
            "lien_page_edition": lien_page,
            "message_accueil_visible": message_accueil_visible,
            "peut_gerer": gestion_logement,
            "peut_gerer_page": gestion_page,
            "api_key": BING_MAPS_API_KEY,
        }

        return render(request, "logement/index.html", c)


def confirm(request):
    if request.ecole != Ecole.n7:
        raise Http404

    if request.session.get("logt_verif_confirm", False):
        del request.session["logt_verif_confirm"]
        request.session["logt_confirm"] = True

    return redirect("logement:accueil")


def detail_by_uuid(request, uuid):
    return detail(request, get_object_or_404(Logement, uuid_proprietaire=uuid), owner=True)


def detail_by_id(request, logement_id):
    return detail(request, get_object_or_404(Logement, pk=logement_id))


def detail(request, logement, owner=False):
    if request.ecole != Ecole.n7:
        raise Http404

    gestion_logement = False
    if not request.user.is_anonymous:
        gestion_logement = request.user.check_permission(request, "gerer_logements")

    if logement.statut in [2, 3] and not owner and not gestion_logement:
        # Annonce supprimée ou en attente de validation et pas les droits
        raise Http404

    c = {
        "logement": logement,
        "logements": [logement],  # pour l'affichage sur la carte
        "peut_gerer": gestion_logement,
        "adr_loca7": EMAIL_LOCA7,
        "api_key": BING_MAPS_API_KEY,
    }

    return render(request, "logement/detail.html", c)


def report_obs(request, logement_id):
    return redirect(f"https://loca7.fr/appartements/{logement_id}/signaler")
    if request.ecole != Ecole.n7:
        raise Http404

    logement = get_object_or_404(Logement, pk=logement_id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            logement.nb_obsolete += 1
            logement.save()

            if logement.nb_obsolete == NB_AVANT_OBSOLETE:
                send_mail(
                    "Annonce de logement obsolète",
                    render_to_string("logement/mail_annonce.txt", locals()),
                    settings.DEFAULT_FROM_EMAIL,
                    [EMAIL_LOCA7],
                    fail_silently=False,
                )

                return redirect("logement:accueil")

    message = """Veuillez confirmer que cette annonce est obsolète."""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Marquer comme obsolète")]

    return confirmation(request, message, boutons)


def contact(request):
    return render(
        request,
        "logement/contact.html",
        {
            "email_gestionnaire": EMAIL_LOCA7,
        },
    )


@permission_needed("gerer_logements")
def liste(request):
    if request.ecole != Ecole.n7:
        raise Http404

    c = {
        "logements": Logement.objects.exclude(statut=2).order_by("-pub_date"),
        "NB_AVANT_OBSOLETE": NB_AVANT_OBSOLETE,
        "corbeille": "non",
    }
    return render(request, "logement/liste.html", c)


@permission_needed("gerer_logements")
def corbeille(request):
    if request.ecole != Ecole.n7:
        raise Http404

    c = {
        "logements": Logement.objects.filter(statut=2).order_by("-pub_date"),
        "NB_AVANT_OBSOLETE": NB_AVANT_OBSOLETE,
        "corbeille": "oui",
    }
    return render(request, "logement/liste.html", c)


@permission_needed("gerer_logements")
def archives(request):
    if request.ecole != Ecole.n7:
        raise Http404

    c = {
        "logements": Logement.objects.filter(statut=1).order_by("-pub_date"),
        "NB_AVANT_OBSOLETE": NB_AVANT_OBSOLETE,
        "corbeille": "oui",
    }
    return render(request, "logement/liste.html", c)


@permission_needed("gerer_logements")
def maj(request):
    if request.ecole != Ecole.n7:
        raise Http404

    for logement in Logement.objects.all():
        if logement.latitude == 0 and logement.longitude == 0:
            logement.maj_coords()
            logement.save()

    return redirect("gestion:logement:liste")


def mail_publication(request, logement):
    email_loca7 = f"Loca7 <{EMAIL_LOCA7}>"
    url = request.build_absolute_uri(logement.get_absolute_url())

    if logement.statut == 3:
        message_admin = render_to_string("logement/mail/message_prepublication_admin.txt", {"url": url})

        msg_admin = EmailMultiAlternatives(
            "Nouvelle Annonce en attente de modération",
            message_admin,
            email_loca7,
            [email_loca7],
        )
        msg_admin.send()
    elif logement.contact_mail:
        # Une @ mail a ete specifiee, on doit envoyer un mail recapitulant l'annonce

        html_url = f'<a href="{url}">{url}</a>'

        message = render_to_string(
            "logement/mail/message_publication.txt",
            {"url": url, "id": logement.id, "uuid": logement.uuid_proprietaire},
        )
        html_msg = render_to_string(
            "logement/mail/message_publication.txt",
            {
                "url": mark_safe(html_url),
                "id": logement.id,
                "uuid": logement.uuid_proprietaire,
            },
        ).replace("\n", "<br />")

        msg = EmailMultiAlternatives(
            "Nouvelle annonce",
            message,
            email_loca7,
            [logement.contact_mail],
            headers={"cc": email_loca7},
        )
        msg.attach_alternative(html_msg, "text/html")
        msg.send()


def editionlogement(request, id=None, exte=True):
    if request.ecole != Ecole.n7:
        raise Http404

    if id is not None:
        if exte:
            raise Http404
        logement = get_object_or_404(Logement, id=id)
        created = False
    else:
        logement = None
        created = True

    if request.method == "POST":
        if not exte:
            form = LogementForm(request.POST, request.FILES, instance=logement)
            photo_form = LogementPhotoFormSet(request.POST, request.FILES, instance=logement)
        else:
            form = LogementPublicForm(request.POST, request.FILES)
            photo_form = LogementPhotoFormSet(request.POST, request.FILES)

        if form.is_valid() and photo_form.is_valid():
            logement = form.save(commit=False)
            logement.nb_obsolete = 0
            if exte:
                logement.statut = 3
            logement.maj_coords()
            logement.save()
            photo_form.instance = logement
            photo_form.save()

            if created:
                mail_publication(request, logement)

            if exte:
                if logement.contact_mail:
                    email_loca7 = f"Loca7 <{EMAIL_LOCA7}>"
                    # Une @ mail a ete specifiee, on doit envoyer un mail notifiant la création de l'annonce
                    message = render_to_string(
                        "logement/mail/message_nouveau.txt",
                        {"url": logement.get_absolute_url(owner=True)},
                    )
                    msg = EmailMultiAlternatives(
                        "Création d'une annonce",
                        message,
                        email_loca7,
                        [logement.contact_mail],
                        headers={"cc": email_loca7},
                    )
                    msg.send()
                return render(request, "logement/fin_nouvelle.html")
            return HttpResponseRedirect(logement.get_absolute_url())
    else:
        if not exte:
            form = LogementForm(instance=logement)
            photo_form = LogementPhotoFormSet(instance=logement)
        else:
            form = LogementPublicForm()
            photo_form = LogementPhotoFormSet()

    c = {
        "form": form,
        "photo_form": photo_form,
        "inscription": not created,
        "l": logement,
    }

    return render(request, "logement/edition.html", c)


def nouveau(request):
    return redirect("https://loca7.fr/appartements/ajouter")
    return editionlogement(request)


@permission_needed("gerer_logements")
def edition(request, id=None):
    return editionlogement(request, id=id, exte=False)


@permission_needed("gerer_logements")
def moderation(request, ident):
    logement = get_object_or_404(Logement, id=ident)
    if logement.statut != 3:
        message = "Cette annonce n'est plus en attente de validation."
        return erreur(request, message, "Pas de modération nécessaire")

    if request.method == "POST":
        if "conf_ok" in request.POST:
            logement.statut = 0
            logement.save()
            mail_publication(request, logement)

        return redirect("logement:detail", ident)

    message = "Êtes-vous sûr de vouloir valider cette annonce ?"
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Valider cette annonce")]
    return confirmation(request, message, boutons, "Valider cette annonce")


@permission_needed("gerer_logements")
def suppression(request, id):
    if request.ecole != Ecole.n7:
        raise Http404

    logement = get_object_or_404(Logement, id=id)

    if logement.statut == 0:
        # Mise à la corbeille
        if request.method != "POST":
            message = "Êtes-vous sûr de vouloir mettre cette annonce à la corbeille ?"
            boutons = [("conf_annul", "Annuler"), ("conf_ok", "Mettre à la corbeille")]
            return confirmation(request, message, boutons, "Supprimer une annonce")

        if "conf_ok" not in request.POST:
            return redirect(logement.get_absolute_url())

        if logement.contact_mail:
            # Une @ mail a ete specifiee, on doit envoyer un mail notifiant la suppression de l'annonce
            email_loca7 = f"Loca7 <{EMAIL_LOCA7}>"
            nom = logement.get_typel_display()
            nom += " #" + str(logement.id)
            message = render_to_string("logement/mail/message_suppression.txt", {"nom": nom})
            msg = EmailMultiAlternatives(
                "Suppression d'une annonce",
                message,
                email_loca7,
                [logement.contact_mail],
                headers={"cc": email_loca7},
            )
            msg.send()

        logement.statut = 2
        logement.save()
        return redirect(logement.get_absolute_url())

    if request.method == "POST":
        if "conf_ok" in request.POST:
            logement.delete()
            return redirect("gestion:logement:liste")
        return redirect(logement.get_absolute_url())

    message = """Êtes-vous sûr de vouloir supprimer cette annonce ?"""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons, "Supprimer une annonce")


def depublication_by_uuid(request, uuid):
    logement = get_object_or_404(Logement, uuid_proprietaire=uuid)
    return depublication(request, logement, owner=True)


@permission_needed("gerer_logements")
def depublication_by_id(request, id):
    logement = get_object_or_404(Logement, id=id)
    return depublication(request, logement)


def depublication(request, logement, owner=False):
    if request.ecole != Ecole.n7:
        raise Http404

    if logement.statut != 0:
        message = "Impossible d'effectuer cette action pour cette annonce"
        return erreur(request, message, "Action Impossible")

    if request.method != "POST":
        message = "Êtes-vous sûr de vouloir dépublier cette annonce ?"
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Dépublier")]
        return confirmation(request, message, boutons, "Dépublier une annonce")

    if "conf_ok" not in request.POST:
        return redirect(logement.get_absolute_url(owner))

    if logement.statut == 0:
        if logement.contact_mail:
            # Une @ mail a ete specifiee, on doit envoyer un mail notifiant la désactivation de l'annonce
            email_loca7 = f"Loca7 <{EMAIL_LOCA7}>"
            nom = logement.get_typel_display()
            nom += " #" + str(logement.id)
            message = render_to_string(
                "logement/mail/message_desactivation.txt",
                {"nom": nom, "uuid": logement.uuid_proprietaire},
            )
            msg = EmailMultiAlternatives(
                "Désactivation d'une annonce",
                message,
                email_loca7,
                [logement.contact_mail],
                headers={"cc": email_loca7},
            )
            msg.send()

        logement.statut = 1
        logement.save()

    return redirect(logement.get_absolute_url(owner))


def republication_by_uuid(request, uuid):
    logement = get_object_or_404(Logement, uuid_proprietaire=uuid)
    return republication(request, logement, owner=True)


@permission_needed("gerer_logements")
def republication_by_id(request, id):
    logement = get_object_or_404(Logement, id=id)
    return republication(request, logement)


def republication(request, logement, owner=False):
    if request.ecole != Ecole.n7:
        raise Http404

    if logement.statut in [0, 3]:
        message = "Impossible d'effectuer cette action pour cette annonce"
        return erreur(request, message, "Action Impossible")

    if request.method != "POST":
        message = "Êtes-vous sûr de vouloir republier cette annonce ?"
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Republier")]
        return confirmation(request, message, boutons, "Republier une annonce")

    if "conf_ok" not in request.POST:
        return redirect(logement.get_absolute_url(owner))

    # Remise en état normal
    logement.statut = 0
    logement.nb_obsolete = 0
    logement.save()
    return redirect(logement.get_absolute_url(owner))


@permission_needed("gerer_logements")
def ajouter_photo(request, logement_id):
    logement = get_object_or_404(Logement, id=logement_id)
    if request.method == "POST":
        form = LogementPhotoForm(request.POST, request.FILES)
        if form.is_valid():
            LogementPhoto(logement=logement, photo=form.cleaned_data["photo"]).save()
            return HttpResponseRedirect(logement.get_absolute_url())
    else:
        form = LogementPhotoForm()

    c = {
        "form": form,
        "logement": logement,
    }

    return render(request, "logement/ajouter_photo.html", c)


@permission_needed("gerer_logements")
def supprimer_anciens(request):
    if "annonce_jalon" in request.GET:
        p = get_object_or_404(Logement, pk=request.GET["annonce_jalon"])

        logements = Logement.objects.filter(pub_date__lte=p.pub_date)
        if request.POST:

            if "confirm" in request.POST and request.POST["confirm"] == "oui":
                logements.delete()
                return redirect("gestion:logement:liste")
        c = {
            "logements": logements,
            "id": p.id,
        }
        return render(request, "logement/supprimer_anciens_confirm.html", c)
    else:

        logements = Logement.objects.order_by("date_maj")
        return render(request, "logement/supprimer_anciens_select.html", {"logements": logements})


@permission_needed("gerer_logements")
def suppression_photo(request, photo_id):
    if request.ecole != Ecole.n7:
        raise Http404

    photo = get_object_or_404(LogementPhoto, id=photo_id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            photo.delete()
        return redirect("gestion:logement:liste")

    message = """Êtes-vous sûr de vouloir supprimer cette photo ? ?"""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons, "Supprimer une photo")
